/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

#include <gtest/gtest.h>
#include <cstdlib> // getenv
#include <string>

#include <cdos/Configuration.h>

class ConfigurationTest : public testing::Test {
protected:
    Configuration _config;

    std::string datapath() const {
        const char* pPath = getenv("TESTDATA_PATH");

        if (pPath == NULL) {
            return "/";
        }
        return std::string(pPath);
    }
};

TEST_F(ConfigurationTest, loadFile) {
    bool loadStatus =
        _config.loadFile(datapath() + "testconfig.json");

    ASSERT_TRUE(loadStatus);

    std::string testvalue = _config.get<std::string>("testkey");
    ASSERT_EQ(testvalue, "testvalue");
}

TEST_F(ConfigurationTest, loadNonexistentFile) {
    bool loadStatus =
        _config.loadFile(datapath() + "nonexistent.json");

    ASSERT_FALSE(loadStatus);
}

TEST_F(ConfigurationTest, getSubtree) {
    _config.loadFile(datapath() + "testconfig.json");
    Configuration subtree = _config.subtree("subtree");

    std::string subvalue = subtree.get<std::string>("subkey");
    ASSERT_EQ(subvalue, "subvalue");
}

TEST_F(ConfigurationTest, size) {
    _config.loadFile(datapath() + "testconfig.json");
    Configuration sized = _config.subtree("size");

    ASSERT_EQ(sized.size(), 3u);
}

TEST_F(ConfigurationTest, iterator) {
    _config.loadFile(datapath() + "testconfig.json");
    Configuration iterable = _config.subtree("iterable");

    Configuration::iterator it = iterable.begin();
    ASSERT_EQ(it->second.data(), "1");
    ++it;
    ASSERT_EQ(it->second.data(), "2");
    ++it;
    ASSERT_EQ(it->second.data(), "3");
    ++it;
    ASSERT_EQ(iterable.end(), it);
}

TEST_F(ConfigurationTest, getUnsigned) {
    _config.loadFile(datapath() + "testconfig.json");

    unsigned u = _config.get<unsigned>("unsigned");
    ASSERT_EQ(42u, u);
}
