/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

#include <gtest/gtest.h>

#include <thread>
#include <utility>

#include <cdos/Queue.hpp>

class QueueTest : public testing::Test {
protected:
    typedef double T;
    typedef Queue<T> Q;

    QueueTest()
        :_qa(1000),
         _qb(1000)
    {}

    Q _qa;
    Q _qb;

    void move(Q& qa, Q& qb) {
        qb.push(qa.pop());
    }

public:
    void moveAB(int moveCount) {
        for (int i = 0; i < moveCount; ++i) {
            move(_qa, _qb);
        }
    }

    void moveBA(int moveCount) {
        for (int i = 0; i < moveCount; ++i) {
            move(_qb, _qa);
        }
    }
};

TEST_F(QueueTest, move) {
    const int itemCount = 10;
    for (double i = 0; i < itemCount; ++i) {
        _qa.push(std::move(i));
    }

    std::thread a1(&QueueTest::moveAB, std::ref(*this), 300);
    std::thread b1(&QueueTest::moveBA, std::ref(*this), 300);
    std::thread a2(&QueueTest::moveAB, std::ref(*this), 300);
    std::thread b2(&QueueTest::moveBA, std::ref(*this), 300);
    std::thread a3(&QueueTest::moveAB, std::ref(*this), 300);
    std::thread b3(&QueueTest::moveBA, std::ref(*this), 300);
    std::thread a4(&QueueTest::moveAB, std::ref(*this), 300);
    std::thread b4(&QueueTest::moveBA, std::ref(*this), 300);
    std::thread a5(&QueueTest::moveAB, std::ref(*this), 300);
    std::thread b5(&QueueTest::moveBA, std::ref(*this), 300);

    a1.join();
    a2.join();
    a3.join();
    a4.join();
    a5.join();

    b1.join();
    b2.join();
    b3.join();
    b4.join();
    b5.join();

    ASSERT_EQ(itemCount, _qa.size() + _qb.size());
}

TEST_F(QueueTest, limitedLength) {
    Q q(10);
    const int itemCount = 100;
    for (double i = 0; i < itemCount; ++i) {
        q.push(std::move(i));
    }

    ASSERT_GE(10, q.size());
    ASSERT_EQ(100, q.size() + q.dropCount());
}
