/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

#include <sstream>

#include <boost/property_tree/json_parser.hpp>
#include <boost/log/trivial.hpp>

#include <cdos/Configuration.h>

Configuration::Configuration(const boost::property_tree::ptree& data)
    :_data(data)
{}

bool Configuration::loadFile(const std::string& fileName) {
	using boost::property_tree::json_parser::read_json;
	using boost::property_tree::json_parser::json_parser_error;

	try {
		read_json(fileName, _data);

	} catch (const json_parser_error& ex) {
		BOOST_LOG_TRIVIAL(warning)
            << "Error while loading file: "
            << ex.what();

		return false;
	}

	return true;
}

bool Configuration::loadString(const std::string& config) {
    using boost::property_tree::json_parser::read_json;
    using boost::property_tree::json_parser::json_parser_error;

    try {
        std::stringstream configStream(config);
        read_json(configStream, _data);
    } catch (const json_parser_error& ex) {
        BOOST_LOG_TRIVIAL(warning)
            << "Error while loading config string: "
            << ex.what();

        return false;
    }

    return true;
}

Configuration Configuration::subtree(const char* key) {
    using boost::property_tree::ptree_bad_path;
    using boost::property_tree::ptree;

    try {
        ptree tree = _data.get_child(key);
        return Configuration(tree);
    } catch (const ptree_bad_path& ex) {
        BOOST_LOG_TRIVIAL(error)
            << "Failed to get subtree: "
            << ex.what();

        return Configuration();
    }
}
