/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

#ifndef PROTOCOL_H_
#define PROTOCOL_H_

#include <boost/asio.hpp>

using boost::asio::ip::tcp;
using boost::asio::ip::udp;

struct TCP {
    typedef tcp base;
    typedef tcp::socket socket;
    typedef tcp::socket request;

    typedef std::function<void(request)> AcceptCb;

    class acceptor {
    public:
        acceptor(
            boost::asio::io_service& ioService,
            tcp::endpoint endpoint
        );

        void accept(socket& socket, const AcceptCb& callback);

        void close(boost::system::error_code& ec);

    private:
        tcp::acceptor _acceptor;
    };

    enum { bufferLength = 1024 };
    enum { maxEmptyResponse = 0 };
};

struct UDP {
    typedef udp base;
    typedef udp::socket socket;

    class request {
    public:
        request();

        request(request&& rhs);

        request(const boost::asio::io_service&);

        void operator=(request&& rhs);

        void connect(base::endpoint endpoint);

        base::endpoint& remote_endpoint();

        boost::asio::mutable_buffers_1 buffer();

        void setSize(std::size_t size);

        std::size_t receive(
            boost::asio::mutable_buffers_1 otherBuffer,
            int, // socket_base::message_flags
            boost::system::error_code& ec
        );

        void send(
            boost::asio::mutable_buffers_1 otherBuffer,
            boost::asio::socket_base::message_flags flags,
            boost::system::error_code& ec
        );

        bool is_open() const;

        void close() {};
        void shutdown(
            base::socket::shutdown_type,
            boost::system::error_code
        ) {}

        enum { maxLength = (1 << 16) - 1 };
    private:
        base::endpoint _remoteEndpoint;
        std::unique_ptr<char[]> _data;
        std::size_t _size;
    };

    typedef std::function<void(request)> AcceptCb;

    class acceptor {
    public:
        acceptor(
            boost::asio::io_service& ioService,
            udp::endpoint endpoint
        );

        void accept(socket& socket, const AcceptCb& callback);

        void close(boost::system::error_code& ec) {}

    private:
        void doAccept(socket& socket, const AcceptCb& callback);

        request _request;
        udp::endpoint _endpoint;
    };

    enum { bufferLength = request::maxLength };
    enum { maxEmptyResponse = 10 };
};


#endif /* PROTOCOL_H_ */
