/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

#include <string>

#include <boost/log/trivial.hpp>

#include <cdos/Service.h>

Service::Service(const Configuration& config)
    :_status(CREATED),
     _publicPort(config.get<unsigned>("publicPort")),
     _privatePort(config.get<unsigned>("privatePort"))
{
    std::string protocol = config.get<std::string>("protocol", "tcp");
    if (protocol == "tcp") {
        _impl = ServiceImplPtr(new ServiceImpl<TCP>(config));
    } else if (protocol == "udp") {
        _impl = ServiceImplPtr(new ServiceImpl<UDP>(config));
    } else {
        throw std::logic_error("Invalid protocol in configuration");
    }
}

void Service::setFilter(FILTER filter) {
    _impl->setFilter(filter);
}

void Service::setBlacklist(const IpFilterPtr& blacklist) {
    _impl->setBlacklist(blacklist);
}

void Service::setWhitelist(const IpFilterPtr& whitelist) {
    _impl->setWhitelist(whitelist);
}

void Service::setCriticalLoad(bool isCritical) {
    _impl->setCriticalLoad(isCritical);
}

void Service::start() {
    switch (_status) {
    case CREATED:
    {
        BOOST_LOG_TRIVIAL(info)
            << "Service starts on ports "
            << _publicPort << " -> " << _privatePort;

        _status = RUNNING;

        _impl->startProcessors();
        _impl->startServer();

        break;
    }
    case RUNNING:
        BOOST_LOG_TRIVIAL(info)
            << "Service is already running";
        break;

    case STOPPED:
        BOOST_LOG_TRIVIAL(warning)
            << "Can't run stopped service";
        break;

    default:
        BOOST_LOG_TRIVIAL(error)
            << "Invalid service status";
        break;
    }
}

void Service::stop() {
    switch (_status) {
    case CREATED:
        BOOST_LOG_TRIVIAL(warning)
            << "Can't stop not running service";
        break;

    case RUNNING:
        BOOST_LOG_TRIVIAL(info)
            << "Stopping service on ports "
            << _publicPort << " -> " << _privatePort;

        _status = STOPPED;

        _impl->stopServer();

        // take care of the processing threads
        // clear load flag
        setCriticalLoad(false);
        _impl->stopProcessors();

        break;

    case STOPPED:
        BOOST_LOG_TRIVIAL(warning)
            << "Can't stop stopped service";
        break;

    default:
        BOOST_LOG_TRIVIAL(error)
            << "Invalid service status";
        break;
    }
}

const Metrics Service::getMetrics() const {
    return _impl->getMetrics();
}
