/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

/**
 * File of Common Definitions
 *
 * To get the definitions related to
 * <feature>, #define REQ_DEF_<FEATURE>
 * before #include <defs.h>
 */

#include <memory>

#ifdef REQ_DEF_FILTER
#ifndef FILTER_DEFINED
#define FILTER_DEFINED

#include <boost/asio/ip/address.hpp>

typedef boost::asio::ip::address IpAddress;

#include <cdos/Filter.hpp>

/**
 * Filtering methods
 */
enum FILTER {
    NO_FILTER, /**< Honor no filter */
    WHITELIST, /**< Use whitelist */
    BLACKLIST  /**< Use blacklist */
};

typedef Filter<IpAddress> IpFilter;
typedef std::shared_ptr<IpFilter> IpFilterPtr;

#endif
#endif


#ifdef REQ_DEF_SERVICE
#ifndef SERVICE_DEFINED
#define SERVICE_DEFINED

#include <cdos/Service.h>

typedef std::shared_ptr<Service> ServicePtr;

#endif
#endif

#ifdef REQ_DEF_METRICS
#ifndef METRICS_DEFINED
#define METRICS_DEFINED
/**
 * Snapshot of Service metrics
 */
struct Metrics {
    /** Count of the accepted connections */
    unsigned totalIncomingRequestCount;

    /** Count of the served connections */
    unsigned totalOutgoingResponseCount;

    /**
     * Count of connections dropped from the
     * the queue because reaching queue size
     * limit
     */
    unsigned totalDroppedFromQueueCount;

    /** Current size of the queue */
    unsigned currentQueueSize;

    /** Count of connections rejected by blacklist */
    unsigned totalBlacklistRejection;

    /** Count of connections rejected by whitelist */
    unsigned totalWhitelistRejection;
};
#endif
#endif
