/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

package CounterDoSJ;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;

/**
 * Wrapper of CounterDoS.so
 * 
 * @author bthaler
 *
 */
class Application {
    static {
        OperatingSystemMXBean mxBean =
            ManagementFactory.getOperatingSystemMXBean();
        
        /**
         * Architecture of the underlying processor
         * which might depend on the JVM bitness (WOW64)
         * 
         * Some possible values:
         *  - i386  (32bit Linux)
         *  - x86   (32bit Windows, 64bit Windows running 32bit JVM)
         *  - amd64 (64bit Linux or Windows)
         *  - sparc (SPARC, not supported)
         *  - ppc   (PowerPC, not supported)
         */
        String arch = mxBean.getArch();
        System.loadLibrary("CounterDoS_" + arch);
    }
    
    public static void start(String configString) {
        System.out.print("Start CDoS application... ");
        
        boolean status = nativeStart(configString);
        if (status) {
            System.out.println("success");
        } else {
            System.out.println("fail");
            System.err.println(nativeGetError());
        }
    }
    
    public static void stop() {
        for (String name : _tunnels.keySet()) {
            nativeStopService(name);
        }
        
        nativeStop();
        _tunnels.clear();
    }
    
    public static boolean startTunnel(Tunnel tunnel) {
        String name = tunnel.getName();
        
        if (_tunnels.containsKey(name)) {
            System.err.println("Tunnel already exists");
            return false;
        }
        
        boolean success = nativeStartService(name, tunnel.getConfig());
        
        if (!success) {
            System.err.println(nativeGetError());
            return false;
        }
        
        _tunnels.put(name, tunnel);
        return success;
    }
    
    public static Tunnel getTunnel(String name) {
        Tunnel tunnel = _tunnels.get(name);
        
        if (tunnel == null) {
            System.err.println("Tunnel doesn't exist");
        }
        
        return tunnel;
    }
    
    public static boolean stopTunnel(Tunnel tunnel) {
        String name = tunnel.getName();
        
        if (_tunnels.containsKey(name) == false) {
            System.err.println("Tunnel doesn't exist");
            return false;
        }
        
        _tunnels.remove(name);
        return nativeStopService(tunnel.getName());
    }
    
    public static boolean updateCriticalLoad(Tunnel tunnel) {
        return nativeUpdateCriticalLoad(tunnel.getName(), tunnel.isCriticalLoad());
    }
    
    public static boolean changeFiltering(Tunnel tunnel) {
        boolean success = nativeChangeFiltering(tunnel.getName(), tunnel.getFilter().asInt());
        if (!success) {
            System.err.println(nativeGetError());
            return false;
        }
        return true;
    }
    
    public static TunnelMetrics getTunnelMetrics(Tunnel tunnel) throws IllegalArgumentException {
        TunnelMetrics metrics = nativeGetServiceMetrics(tunnel.getName());
        
        if (metrics == null) {
            throw new IllegalArgumentException(nativeGetError());
        }
        
        return metrics;
    }
    
    public static boolean blacklistIp(InetAddress ip) {
        boolean success = nativeBlockIp(ip.getHostAddress());
        if (!success) {
            System.err.println(nativeGetError());
        }
        
        return success;
    }
    
    public static boolean unblacklistIp(InetAddress ip) {
        boolean success = nativeUnblockIp(ip.getHostAddress());
        if (!success) {
            System.err.println(nativeGetError());
        }
        
        return success;
    }
    
    public static boolean whitelistIp(InetAddress ip) {
        boolean success = nativeWhitelistIp(ip.getHostAddress());
        if (!success) {
            System.err.println(nativeGetError());
        }
        
        return success;
    }
    
    public static boolean unwhitelistIp(InetAddress ip) {
        boolean success = nativeUnwhitelistIp(ip.getHostAddress());
        if (!success) {
            System.err.println(nativeGetError());
        }
        
        return success;
    }
    
    private static Map<String, Tunnel> _tunnels = new HashMap<String, Tunnel>();
    
    private static native String nativeGetError();
    
    private static native boolean nativeStart(String configString);
    private static native boolean nativeStop();
    
    private static native boolean nativeStartService(String name, String jsonConfig);
    private static native boolean nativeStopService(String name);
    
    private static native boolean nativeUpdateCriticalLoad(String serviceName, boolean isCritical);
    private static native boolean nativeChangeFiltering(String serviceName, int filteringMethod);
    
    private static native TunnelMetrics nativeGetServiceMetrics(String serviceName);
    
    private static native boolean nativeBlockIp(String ip);
    private static native boolean nativeUnblockIp(String ip);
    
    private static native boolean nativeWhitelistIp(String ip);
    private static native boolean nativeUnwhitelistIp(String ip);
}
