/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

package CounterDoSJ;

import java.io.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;

import org.knopflerfish.service.console.*;

import CounterDoSJ.Tunnel.FILTER;
import CounterDoSJ.Tunnel.PROTOCOL;

public class CDoSCommandGroup extends CommandGroupAdapter {

    CDoSCommandGroup() {
        super("cdos", "Control CounterDoS bundle");
    }
    
    // START TUNNEL
    public final static String USAGE_START = "<name> <ip> <publicPort> <privatePort> [<protocol>]";
    
    public final static String[] HELP_START = new String[] {
        "Starts a tunnel",
        "<name>\tName of the tunnel to start",
        "<ip>\tIP of private service",
        "<publicPort>\tPublic port to protect",
        "<privatePort>\tPort of private service",
        "[<protocol>]\tProtocol, tcp or udp"
    };
    
    public int cmdStart(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String name = (String) options.get("name");
        String ipStr = (String) options.get("ip");
        int publicPort = Integer.parseInt((String)options.get("publicPort"));
        int privatePort = Integer.parseInt((String)options.get("privatePort"));
        String protocolStr = (String) options.get("protocol");
        
        PROTOCOL protocol;
        if (protocolStr == null) {
            protocolStr = "tcp";
        }
        
        switch (protocolStr) {
        case "tcp":
            protocol = PROTOCOL.TCP;
            break;
        case "udp":
            protocol = PROTOCOL.UDP;
            break;
        default:
            System.err.println("Invalid protocol:" + protocolStr);
            return 1;
        }
        
        InetAddress ip;
        try {
            ip = InetAddress.getByName(ipStr);
        } catch (UnknownHostException e) {
            System.err.println("Invalid IP:" + ipStr);
            return 1;
        }
        
        Tunnel tunnel = new Tunnel(name, ip, publicPort, privatePort, protocol);
        
        System.out.println("Create tunnel:");
        System.out.println(tunnel.getConfig());
        
        tunnel.start();
        
        return 0;
    }
    
    // STOP TUNNEL
    public final static String USAGE_STOP = "<tunnel>";
    
    public final static String[] HELP_STOP = new String[] {
        "Stops a tunnel",
        "<tunnel>\tName of the tunnel to stop"
    };
    
    public int cmdStop(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String name = (String) options.get("tunnel");
        Tunnel tunnel = Application.getTunnel(name);
        if (tunnel == null) {
            System.err.println("Failed to stop tunnel");
            return 1;
        }
        
        tunnel.stop();
        
        return 0;
    }
    
    // SET CRITICAL LOAD
    public final static String USAGE_CRITICAL = "<tunnelName> <isCritical>";
    
    public final static String[] HELP_CRITICAL = new String[] {
        "Sets critical load flag of the tunnel",
        "<tunnelName>\tName of the tunnel",
        "<isCritical>\tBoolean flag, true or false"
    };
    
    public int cmdCritical(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String name = (String) options.get("tunnelName");
        boolean isCritical = Boolean.valueOf((String) options.get("isCritical"));
        
        Tunnel tunnel = Application.getTunnel(name);
        if (tunnel == null) {
            System.err.println("Tunnel not found");
            return 1;
        }
        
        tunnel.updateCriticalLoad(isCritical);
        
        return 0;
    }
    
    // SET FILTERING
    public final static String USAGE_FILTER = "<tunnelName> <filtering>";
    
    public final static String[] HELP_FILTER = new String[] {
        "Sets filtering method of tunnel",
        "<tunnelName>\tName of the tunnel",
        "<filtering>\tAvailable methods: nofilter, white, black"
    };
    
    public int cmdFilter(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String name = (String) options.get("tunnelName");
        String filterStr = (String) options.get("filtering");
        
        Tunnel tunnel = Application.getTunnel(name);
        if (tunnel == null) {
            System.err.println("Tunnel not found");
            return 1;
        }
        
        Tunnel.FILTER filter;
        switch (filterStr) {
        case "nofilter":
            filter = FILTER.NO_FILTER;
            break;
        case "white":
            filter = FILTER.WHITELIST;
            break;
        case "black":
            filter = FILTER.BLACKLIST;
            break;
        default:
            System.err.println("Invalid filtering");
            return 1;
        }
        
        tunnel.changeFiltering(filter);
        
        return 0;
    }
    
    
    // DEMO
    public final static String USAGE_DEMO = "";
    
    public final static String[] HELP_DEMO = new String[] {
        "Launches demo tunnel and load watcher"
    };
    
    public int cmdDemo(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        System.out.println("Start demo tunnel 'apache' on ports: 8090 -> 80");
        
        try {
            InetAddress privateServiceAddr = InetAddress.getByName("127.0.0.1");
            Tunnel tunnel = new Tunnel("apache", privateServiceAddr, 8090, 80);
            
            System.out.println(tunnel.getConfig());
            
            tunnel.start();
            
            // add load watcher
            LoadWatcher watcher = new LoadWatcher(tunnel);
            watcher.run();
            
        } catch (UnknownHostException e) {
            System.err.println("Failed to create localhost address");
            return 1;
        }
        
        return 0;
    }

    // BLOCK
    public final static String USAGE_BLOCK = "<ip>";

    public final static String[] HELP_BLOCK = new String[] {
        "Blacklist ip address", 
        "<ip> IP address to block" 
    };

    public int cmdBlock(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String ipStr = (String) options.get("ip");
        InetAddress ip;
        
        try {
            ip = InetAddress.getByName(ipStr);
        } catch (UnknownHostException e) {
            System.err.println("Invalid IP address.");
            return 1;
        }
        
        Application.blacklistIp(ip);
        
        return 0;
    }
    
    // UNBLOCK
    public final static String USAGE_UNBLOCK = "<ip>";

    public final static String[] HELP_UNBLOCK = new String[] {
        "Remove IP address from the blacklst", 
        "<ip> IP address to unblock" 
    };

    public int cmdUnblock(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String ipStr = (String) options.get("ip");
        InetAddress ip;
        
        try {
            ip = InetAddress.getByName(ipStr);
        } catch (UnknownHostException e) {
            System.err.println("Invalid IP address.");
            return 1;
        }
        
        Application.unblacklistIp(ip);
        
        return 0;
    }
    
    // TRUST
    public final static String USAGE_TRUST = "<ip>";

    public final static String[] HELP_TRUST = new String[] {
        "Whitelist ip address", 
        "<ip> IP address to whitelist" 
    };

    public int cmdTrust(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String ipStr = (String) options.get("ip");
        InetAddress ip;
        
        try {
            ip = InetAddress.getByName(ipStr);
        } catch (UnknownHostException e) {
            System.err.println("Invalid IP address.");
            return 1;
        }
        
        Application.whitelistIp(ip);
        
        return 0;
    }
    
    // UNTRUST
    public final static String USAGE_UNTRUST = "<ip>";

    public final static String[] HELP_UNTRUST = new String[] {
        "Remove IP address from the whitelist", 
        "<ip> IP address to unwhitelist" 
    };

    public int cmdUntrust(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String ipStr = (String) options.get("ip");
        InetAddress ip;
        
        try {
            ip = InetAddress.getByName(ipStr);
        } catch (UnknownHostException e) {
            System.err.println("Invalid IP address.");
            return 1;
        }
        
        Application.unwhitelistIp(ip);
        
        return 0;
    }
    
    // METRICS
    public final static String USAGE_METRICS = "<tunnelName>";
    
    public final static String[] HELP_METRICS = new String[] {
        "Get tunnel metrics",
        "<tunnelName>\tName of the tunnel"
    };
    
    public int cmdMetrics(
        Dictionary options, 
        Reader in, 
        PrintWriter out,
        Session session
    ) {
        String name = (String) options.get("tunnelName");
        Tunnel tunnel = Application.getTunnel(name);
        if (tunnel == null) {
            System.err.println("Tunnel not found");
            return 1;
        }
        
        try {
            TunnelMetrics metrics = tunnel.getMetrics();
            System.out.println(metrics.toString());
        } catch (IllegalArgumentException ex) {
            System.err.println("Failed to get metrics: " + ex.getMessage());
            return 1;
        }
        
        return 0;
    }
}