/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

package CounterDoSJ;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import CounterDoSJ.Tunnel.FILTER;


class LoadWatcher extends Thread {
    public LoadWatcher(Tunnel tunnel) {
        _tunnel = tunnel;
    }
    
    public void run() {
        OperatingSystemMXBean mxBean =
            ManagementFactory.getOperatingSystemMXBean();
        
        int maxLoad = mxBean.getAvailableProcessors();
        System.out.println(String.format("LW: Processor count: %d", maxLoad));

        boolean prevCritical = false;
        
        _running = true;
        _runningWatchers.add(this);
        
        while (_running) {
            double load = mxBean.getSystemLoadAverage();
            
            if (load < 0) {
                System.err.println(
                    "OperatingSystemMXBean.getSystemLoadAverage is not implemented on this platform, LoadWatcher stops"
                );
                halt();
            }
            
            System.out.println(String.format("LW: load: %.2f", load));
            
            boolean isCritical = load > maxLoad;
            
            if (isCritical != prevCritical) {
                if (isCritical) {
                    System.out.println("LW: Critical load detected");
                    _tunnel.changeFiltering(FILTER.WHITELIST);
                } else {
                    System.out.println("LW: Critical load cleared");
                    _tunnel.changeFiltering(FILTER.BLACKLIST);
                }
                
                _tunnel.updateCriticalLoad(isCritical);
            }
            
            prevCritical = isCritical;
            
            try {
                sleep(2000);
            } catch (InterruptedException e) { /* let it pass */ }
        }
    }
    
    public void halt() {
        _running = false;
        _runningWatchers.remove(this);
    }
    
    public synchronized static void haltAll() {
        for (LoadWatcher lw : _runningWatchers) {
            lw._running = false;
        }
        
        _runningWatchers.clear();
    }
    
    private boolean _running;
    private Tunnel _tunnel;
    
    // Thread management
    private static List<LoadWatcher> _runningWatchers = 
        Collections.synchronizedList(new ArrayList<LoadWatcher>());
}
