/*
 * Copyright (c) 2013 SEARCH-LAB Ltd.
 * 
 */

package CounterDoSJ;

import java.net.InetAddress;

public class Tunnel {   
    public Tunnel(String name, InetAddress ip, int publicPort, int privatePort) 
        throws IllegalArgumentException
    {
        this(name, ip, publicPort, privatePort, PROTOCOL.TCP);
    }
    
    public Tunnel(String name, InetAddress ip, int publicPort, int privatePort, PROTOCOL protocol) 
        throws IllegalArgumentException
    {
        _status = STATUS.CREATED;
        _name = name;
        _ip = ip;
        _publicPort = publicPort;
        _privatePort = privatePort;
        _protocol = protocol;
    }
    
    private enum STATUS {
        CREATED,
        RUNNING,
        STOPPED
    }
    
    public void start() {
        boolean status = Application.startTunnel(this);
        if (status) {
            _status = STATUS.RUNNING;
        } else {
            System.err.println("Failed to start tunnel");
        }
    }
    
    public void stop() {
        boolean status = Application.stopTunnel(this);
        if (status) {
            _status = STATUS.STOPPED;
        } else {
            System.err.println("Failed to stop tunnel");
        }
    }
    
    private STATUS _status;
    private String _name;
    private InetAddress _ip;
    private int _publicPort;
    private int _privatePort;
    private int _maxConcurrentClient = 2;
    private int _maxQueueSize = 1000;
    
    private boolean _isCriticalLoad = false;
    private FILTER _currentFilter = FILTER.BLACKLIST;
    private PROTOCOL _protocol = PROTOCOL.TCP;
    
    public String getName() {
        return _name;
    }
    
    public InetAddress getIp() {
        return _ip;
    }

    public int getPublicPort() {
        return _publicPort;
    }

    public int getPrivatePort() {
        return _privatePort;
    }
    
    public void setMaxConcurrentClient(int maxConcurrentClient) {
        _maxConcurrentClient = maxConcurrentClient;
    }

    public int getMaxConcurrentClient() {
        return _maxConcurrentClient;
    }
    
    public int getMaxQueueSize() {
        return _maxQueueSize;
    }

    public void setMaxQueueSize(int maxQueueSize) {
        this._maxQueueSize = maxQueueSize;
    }
    
    public boolean isCriticalLoad() {
        return _isCriticalLoad;
    }

    public void updateCriticalLoad(boolean isCriticalLoad) {
        if (_status != STATUS.RUNNING) {
            System.err.println("Tunnel is not running, can't update load flag");
            return;
        }
        
        _isCriticalLoad = isCriticalLoad;
        Application.updateCriticalLoad(this);
    }
    
    public enum FILTER {
        NO_FILTER(0), /**< Honor no filter */
        WHITELIST(1), /**< Use whitelist */
        BLACKLIST(2);  /**< Use blacklist */
        
        private final int value;
        
        private FILTER(int value) {
            this.value = value;
        }
        
        public int asInt() {
            return value;
        }
    }
    
    public void changeFiltering(FILTER filter) {
        _currentFilter = filter;
        Application.changeFiltering(this);
    }
    
    public FILTER getFilter() {
        return _currentFilter;
    }
    
    public enum PROTOCOL {
        TCP,
        UDP;
        
        public String asString() {
            return toString().toLowerCase();
        }
    }
    
    public String getConfig() {
        return "{"
            + "\"protocol\" : \"" + _protocol.asString() + "\","
            + "\"ipAddress\" : \"" + _ip.getHostAddress() + "\","
            + "\"publicPort\" : " + _publicPort + ","
            + "\"privatePort\" : " + _privatePort + ","
            + "\"maxConcurrentClient\" : " + _maxConcurrentClient + ","
            + "\"maxQueueSize\" : " + _maxQueueSize
            + "}";
    }
    
    public TunnelMetrics getMetrics() throws IllegalArgumentException {
        return Application.getTunnelMetrics(this);
    }
}
