package CounterDoSJTest.network.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketTimeoutException;

import CounterDoSJTest.test.Util;

public class TCPClient extends Client {

    public TCPClient(InetAddress clientAddress, InetAddress serverAddress, int serverPort) {
        super(clientAddress, serverAddress, serverPort);
    }

    @Override
    public boolean testEcho() {
        Socket echoSocket = null;
        PrintWriter out = null;
        BufferedReader in = null;

        try {
            echoSocket = new Socket(_serverAddress, _serverPort, _clientAddress, 0);
            echoSocket.setSoTimeout(1000);
            
            out = new PrintWriter(echoSocket.getOutputStream(), true);
            in = new BufferedReader(
                new InputStreamReader(echoSocket.getInputStream())
            );

            String message = "hello echo server";
            
            Util.log("testEcho writes socket");
            out.println(message);
            
            Util.log("testEcho reads socket");
            String messageEcho = in.readLine();
            
            return Util.expectEquals(message, messageEcho);
        
        } catch (IOException ex) {
            ex.printStackTrace();
            
            return false;
        } finally {
            out.close();
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            
            try {
                echoSocket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public boolean testSilence() {
        Socket echoSocket = null;
        PrintWriter out = null;
        BufferedReader in = null;

        try {
            echoSocket = new Socket(_serverAddress, _serverPort, _clientAddress, 0);
            echoSocket.setSoTimeout(1000);
            
            out = new PrintWriter(echoSocket.getOutputStream(), true);
            in = new BufferedReader(
                new InputStreamReader(echoSocket.getInputStream())
            );

            String message = "hello echo server, I may expect no answer I guess.";
            
            Util.log("testSilence writes socket");
            out.println(message);

            try {
                Util.log("testSilence reads socket");
                String messageEcho = in.readLine();
                
                if (messageEcho == null) {
                    Util.log("Null answer, OK");
                    return true;
                } else {
                    Util.log("FAILURE, silence was expected, got: " + messageEcho);
                }
                
            } catch (SocketTimeoutException ex) {
                // expected timeout
                Util.log("No answer, OK");
                return true;
            }
            
            // silence was expected
            return false;
        
        } catch (IOException ex) {
            ex.printStackTrace();
            
            return false;
        } finally {
            out.close();
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            
            try {
                echoSocket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
