package CounterDoSJTest.network.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.SocketTimeoutException;

import CounterDoSJTest.test.Util;

public class TCPServer extends Server {

    public TCPServer(int port) throws IOException {
        super(port);
        
        _serverSocket = new ServerSocket(_port);
    }

    @Override
    public synchronized void listen() {
        Util.log("TCPServer listens");
        
        if (_status != STATUS.CREATED && _status != STATUS.STOPPED) {
            Util.log("TCPServer: try to listen() in invalid state");
            return;
        }
        
        _status = STATUS.RUNNING;
        
        _responder = new EchoResponder();
        _responder.start();
    }
    
    @Override
    public synchronized void stop() {
        Util.log("TCPServer stops");
        
        if (_status != STATUS.RUNNING) {
            Util.log("TCPServer is not running, can't stop");
            return;
        }
        
        _status = STATUS.STOPPING;
        
        try {
            _responder.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        try {
            _serverSocket.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        _status = STATUS.STOPPED;
    }
    
    private class EchoResponder extends Thread {
        public EchoResponder() {
            super("TCP EchoResponder");
        }

        public void run() {
            try {
                _serverSocket.setSoTimeout(1000);
            } catch (SocketException e) {
                e.printStackTrace();
                return;
            }
            
            while (_status == STATUS.RUNNING) {
                try {
                    accept(_serverSocket.accept());
                } catch (SocketTimeoutException ex) {
                    // expected timeout
                    // let it pass
                } catch (IOException e) {
                    e.printStackTrace();
                    break;
                }
            }
        }
        
        private void accept(Socket acceptedSocket) {
            Util.log("EchoResponder accepts socket");
            
            try {
                acceptedSocket.setSoTimeout(1000);
            } catch (SocketException e) {
                e.printStackTrace();
                return;
            }
            
            PrintWriter out = null;
            BufferedReader in = null;
            
            try {
                out = new PrintWriter(
                    acceptedSocket.getOutputStream(), 
                    true
                );
                
                in = new BufferedReader(
                    new InputStreamReader(acceptedSocket.getInputStream())
                );
    
                String inputLine;
    
                Util.log("EchoResponder reads socket");
                while ((inputLine = in.readLine()) != null) {
                    Util.log("EchoResponder writes socket");
                    out.println(inputLine);
                }
                
            } catch (SocketTimeoutException ex) {
                ex.printStackTrace();
            } catch (IOException ex) {
                ex.printStackTrace();
            } finally {
                if (out != null) {
                    out.close();
                }
                
                try {
                    if (in != null) {
                        in.close();
                    }
                    
                    acceptedSocket.close();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }
    
    private ServerSocket _serverSocket;
    private EchoResponder _responder;
}
