package CounterDoSJTest.network.server;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketTimeoutException;

import CounterDoSJTest.test.Util;

public class UDPServer extends Server {

    public UDPServer(int port) throws IOException {
        super(port);
        
        _serverSocket = new DatagramSocket(_port);
    }

    @Override
    public void listen() {
        Util.log("UDPServer listens");
        
        if (_status != STATUS.CREATED && _status != STATUS.STOPPED) {
            Util.log("UDPServer: try to listen() in invalid state");
            return;
        }
        
        _status = STATUS.RUNNING;
        
        _responder = new EchoResponder();
        _responder.start();
    }

    @Override
    public void stop() {
        Util.log("UDPServer stops");
        
        if (_status != STATUS.RUNNING) {
            Util.log("UDPServer is not running, can't stop");
            return;
        }
        
        _status = STATUS.STOPPING;
        
        try {
            _responder.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        _serverSocket.close();
        
        _status = STATUS.STOPPED;
    }
    
    private class EchoResponder extends Thread {
        public EchoResponder() {
            super("UDP EchoResponder");
        }

        public void run() {
            while (_status == STATUS.RUNNING) {
                try {
                    respond();
                } catch (SocketTimeoutException ex) {
                    // let it pass
                } catch (IOException ex) {
                    ex.printStackTrace();
                    break;
                }
            }
        }
        
        private void respond() throws SocketTimeoutException, IOException {
            byte[] dataIn = new byte[1 << 16];
            
            DatagramPacket packetIn = new DatagramPacket(dataIn, dataIn.length);
            
            _serverSocket.setSoTimeout(1000);
            
            Util.log("EchoResponder reads socket");
            _serverSocket.receive(packetIn);
            
            InetAddress clientIp = packetIn.getAddress();
            int clientPort = packetIn.getPort();
            
            DatagramPacket packetOut = new DatagramPacket(
                dataIn, packetIn.getLength(), clientIp, clientPort
            );
            
            Util.log("EchoResponder writes socket (" 
                + packetIn.getLength() + 
                " bytes) to remote port " + clientPort
            );
            
            _serverSocket.send(packetOut);
        }
    }
    
    private DatagramSocket _serverSocket;
    private EchoResponder _responder;
}